@@ -1,5 +1,5 @@
 /* Iterate over arguments from argv or --files0-from=FILE
-   Copyright (C) 2008-2010 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,10 +12,14 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+
+/* This file uses _GL_ATTRIBUTE_DEALLOC, _GL_ATTRIBUTE_PURE.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
 
 #include <stdio.h>
-#include <stdbool.h>
 
 /* Definition of _GL_ARG_NONNULL.  */
 #include "arg-nonnull.h"
@@ -30,13 +34,14 @@ enum argv_iter_err
   AI_ERR_READ
 };
 
+void argv_iter_free (struct argv_iterator *)
+  _GL_ARG_NONNULL ((1));
+
 struct argv_iterator *argv_iter_init_argv (char **argv)
-  _GL_ARG_NONNULL ((1));
+  _GL_ARG_NONNULL ((1)) _GL_ATTRIBUTE_DEALLOC (argv_iter_free, 1);
 struct argv_iterator *argv_iter_init_stream (FILE *fp)
-  _GL_ARG_NONNULL ((1));
+  _GL_ARG_NONNULL ((1)) _GL_ATTRIBUTE_DEALLOC (argv_iter_free, 1);
 char *argv_iter (struct argv_iterator *, enum argv_iter_err *)
   _GL_ARG_NONNULL ((1, 2));
 size_t argv_iter_n_args (struct argv_iterator const *)
-  _GL_ARG_NONNULL ((1));
-void argv_iter_free (struct argv_iterator *)
-  _GL_ARG_NONNULL ((1));
+  _GL_ATTRIBUTE_PURE _GL_ARG_NONNULL ((1));
