@@ -1,5 +1,5 @@
-/* root-dev-ino.c -- get the device and inode numbers for `/'.
-   Copyright (C) 2003, 2005, 2006, 2009 Free Software Foundation, Inc.
+/* root-dev-ino.c -- get the device and inode numbers for '/'.
+   Copyright (C) 2003-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Jim Meyering. */
 
@@ -22,15 +22,15 @@
 
 #include <stdlib.h>
 
-/* Call lstat to get the device and inode numbers for `/'.
-   Upon failure, return NULL.  Otherwise, set the members of
+/* Call lstat to get the device and inode numbers for '/'.
+   Upon failure, return nullptr.  Otherwise, set the members of
    *ROOT_D_I accordingly and return ROOT_D_I.  */
 struct dev_ino *
 get_root_dev_ino (struct dev_ino *root_d_i)
 {
   struct stat statbuf;
   if (lstat ("/", &statbuf))
-    return NULL;
+    return nullptr;
   root_d_i->st_ino = statbuf.st_ino;
   root_d_i->st_dev = statbuf.st_dev;
   return root_d_i;
