@@ -1,5 +1,5 @@
 /* join - join lines of two files on a common field
-   Copyright (C) 1991-2014 Free Software Foundation, Inc.
+   Copyright (C) 1991-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,18 +12,17 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    Written by Mike Haertel, mike@gnu.ai.mit.edu.  */
 
 #include <config.h>
 
-#include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
 
 #include "system.h"
-#include "error.h"
+#include "assure.h"
 #include "fadvise.h"
 #include "hard-locale.h"
 #include "linebuffer.h"
@@ -56,7 +55,7 @@ struct outlist
     int file;
 
     /* Field index (zero-based), specified only when FILE is 1 or 2.  */
-    size_t field;
+    idx_t field;
 
     struct outlist *next;
   };
@@ -65,15 +64,15 @@ struct outlist
 struct field
   {
     char *beg;			/* First character in field.  */
-    size_t len;			/* The length of the field.  */
+    idx_t len;			/* The length of the field.  */
   };
 
 /* A line read from an input file.  */
 struct line
   {
     struct linebuffer buf;	/* The line itself.  */
-    size_t nfields;		/* Number of elements in 'fields'.  */
-    size_t nfields_allocated;	/* Number of elements allocated for 'fields'. */
+    idx_t nfields;		/* Number of elements in 'fields'.  */
+    idx_t nfields_allocated;	/* Number of elements allocated for 'fields'. */
     struct field *fields;
   };
 
@@ -81,13 +80,13 @@ struct line
    same join field value.  */
 struct seq
   {
-    size_t count;			/* Elements used in 'lines'.  */
-    size_t alloc;			/* Elements allocated in 'lines'.  */
+    idx_t count;		/* Elements used in 'lines'.  */
+    idx_t alloc;		/* Elements allocated in 'lines'.  */
     struct line **lines;
   };
 
 /* The previous line read from each file.  */
-static struct line *prevline[2] = {NULL, NULL};
+static struct line *prevline[2] = {nullptr, nullptr};
 
 /* The number of lines read from each file.  */
 static uintmax_t line_no[2] = {0, 0};
@@ -98,7 +97,7 @@ static char *g_names[2];
 /* This provides an extra line buffer for each file.  We need these if we
    try to read two consecutive lines into the same buffer, since we don't
    want to overwrite the previous buffer before we check order. */
-static struct line *spareline[2] = {NULL, NULL};
+static struct line *spareline[2] = {nullptr, nullptr};
 
 /* True if the LC_COLLATE locale is hard.  */
 static bool hard_LC_COLLATE;
@@ -122,12 +121,12 @@ static char const *empty_filler;
 static bool autoformat;
 /* The number of fields to output for each line.
    Only significant when autoformat is true.  */
-static size_t autocount_1;
-static size_t autocount_2;
+static idx_t autocount_1;
+static idx_t autocount_2;
 
-/* Field to join on; SIZE_MAX means they haven't been determined yet.  */
-static size_t join_field_1 = SIZE_MAX;
-static size_t join_field_2 = SIZE_MAX;
+/* Field to join on; -1 means they haven't been determined yet.  */
+static ptrdiff_t join_field_1 = -1;
+static ptrdiff_t join_field_2 = -1;
 
 /* List of fields to print.  */
 static struct outlist outlist_head;
@@ -158,14 +157,14 @@ enum
 
 static struct option const longopts[] =
 {
-  {"ignore-case", no_argument, NULL, 'i'},
-  {"check-order", no_argument, NULL, CHECK_ORDER_OPTION},
-  {"nocheck-order", no_argument, NULL, NOCHECK_ORDER_OPTION},
-  {"zero-terminated", no_argument, NULL, 'z'},
-  {"header", no_argument, NULL, HEADER_LINE_OPTION},
+  {"ignore-case", no_argument, nullptr, 'i'},
+  {"check-order", no_argument, nullptr, CHECK_ORDER_OPTION},
+  {"nocheck-order", no_argument, nullptr, NOCHECK_ORDER_OPTION},
+  {"zero-terminated", no_argument, nullptr, 'z'},
+  {"header", no_argument, nullptr, HEADER_LINE_OPTION},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 /* Used to print non-joining lines */
@@ -194,31 +193,41 @@ Usage: %s [OPTION]... FILE1 FILE2\n\
               program_name);
       fputs (_("\
 For each pair of input lines with identical join fields, write a line to\n\
-standard output.  The default join field is the first, delimited\n\
-by whitespace.  When FILE1 or FILE2 (not both) is -, read standard input.\n\
+standard output.  The default join field is the first, delimited by blanks.\
 \n\
-  -a FILENUM        also print unpairable lines from file FILENUM, where\n\
-                      FILENUM is 1 or 2, corresponding to FILE1 or FILE2\n\
-  -e EMPTY          replace missing input fields with EMPTY\n\
 "), stdout);
       fputs (_("\
-  -i, --ignore-case  ignore differences in case when comparing fields\n\
-  -j FIELD          equivalent to '-1 FIELD -2 FIELD'\n\
-  -o FORMAT         obey FORMAT while constructing output line\n\
-  -t CHAR           use CHAR as input and output field separator\n\
+\n\
+When FILE1 or FILE2 (not both) is -, read standard input.\n\
 "), stdout);
       fputs (_("\
-  -v FILENUM        like -a FILENUM, but suppress joined output lines\n\
-  -1 FIELD          join on this FIELD of file 1\n\
-  -2 FIELD          join on this FIELD of file 2\n\
-  --check-order     check that the input is correctly sorted, even\n\
-                      if all input lines are pairable\n\
-  --nocheck-order   do not check that the input is correctly sorted\n\
-  --header          treat the first line in each file as field headers,\n\
-                      print them without trying to pair them\n\
+\n\
+  -a FILENUM             also print unpairable lines from file FILENUM, where\n\
+                           FILENUM is 1 or 2, corresponding to FILE1 or FILE2\n\
 "), stdout);
       fputs (_("\
-  -z, --zero-terminated     line delimiter is NUL, not newline\n\
+  -e STRING              replace missing (empty) input fields with STRING;\n\
+                           I.e., missing fields specified with '-12jo' options\
+\n\
+"), stdout);
+      fputs (_("\
+  -i, --ignore-case      ignore differences in case when comparing fields\n\
+  -j FIELD               equivalent to '-1 FIELD -2 FIELD'\n\
+  -o FORMAT              obey FORMAT while constructing output line\n\
+  -t CHAR                use CHAR as input and output field separator\n\
+"), stdout);
+      fputs (_("\
+  -v FILENUM             like -a FILENUM, but suppress joined output lines\n\
+  -1 FIELD               join on this FIELD of file 1\n\
+  -2 FIELD               join on this FIELD of file 2\n\
+      --check-order      check that the input is correctly sorted, even\n\
+                           if all input lines are pairable\n\
+      --nocheck-order    do not check that the input is correctly sorted\n\
+      --header           treat the first line in each file as field headers,\n\
+                           print them without trying to pair them\n\
+"), stdout);
+      fputs (_("\
+  -z, --zero-terminated  line delimiter is NUL, not newline\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
@@ -239,7 +248,7 @@ Note, comparisons honor the rules specified by 'LC_COLLATE'.\n\
 If the input is not sorted and some lines cannot be joined, a\n\
 warning message will be given.\n\
 "), stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
@@ -247,12 +256,11 @@ warning message will be given.\n\
 /* Record a field in LINE, with location FIELD and size LEN.  */
 
 static void
-extract_field (struct line *line, char *field, size_t len)
+extract_field (struct line *line, char *field, idx_t len)
 {
   if (line->nfields >= line->nfields_allocated)
-    {
-      line->fields = X2NREALLOC (line->fields, &line->nfields_allocated);
-    }
+    line->fields = xpalloc (line->fields, &line->nfields_allocated, 1,
+                            -1, sizeof *line->fields);
   line->fields[line->nfields].beg = field;
   line->fields[line->nfields].len = len;
   ++(line->nfields);
@@ -272,25 +280,25 @@ xfields (struct line *line)
   if (0 <= tab && tab != '\n')
     {
       char *sep;
-      for (; (sep = memchr (ptr, tab, lim - ptr)) != NULL; ptr = sep + 1)
+      for (; (sep = memchr (ptr, tab, lim - ptr)) != nullptr; ptr = sep + 1)
         extract_field (line, ptr, sep - ptr);
     }
   else if (tab < 0)
     {
       /* Skip leading blanks before the first field.  */
-      while (isblank (to_uchar (*ptr)))
+      while (field_sep (*ptr))
         if (++ptr == lim)
           return;
 
       do
         {
           char *sep;
-          for (sep = ptr + 1; sep != lim && ! isblank (to_uchar (*sep)); sep++)
+          for (sep = ptr + 1; sep != lim && ! field_sep (*sep); sep++)
             continue;
           extract_field (line, ptr, sep - ptr);
           if (sep == lim)
             return;
-          for (ptr = sep + 1; ptr != lim && isblank (to_uchar (*ptr)); ptr++)
+          for (ptr = sep + 1; ptr != lim && field_sep (*ptr); ptr++)
             continue;
         }
       while (ptr != lim);
@@ -302,12 +310,12 @@ xfields (struct line *line)
 static void
 freeline (struct line *line)
 {
-  if (line == NULL)
+  if (line == nullptr)
     return;
   free (line->fields);
-  line->fields = NULL;
+  line->fields = nullptr;
   free (line->buf.buffer);
-  line->buf.buffer = NULL;
+  line->buf.buffer = nullptr;
 }
 
 /* Return <0 if the join field in LINE1 compares less than the one in LINE2;
@@ -317,14 +325,14 @@ freeline (struct line *line)
 
 static int
 keycmp (struct line const *line1, struct line const *line2,
-        size_t jf_1, size_t jf_2)
+        idx_t jf_1, idx_t jf_2)
 {
   /* Start of field to compare in each file.  */
   char *beg1;
   char *beg2;
 
-  size_t len1;
-  size_t len2;		/* Length of fields to compare.  */
+  idx_t len1;
+  idx_t len2;		/* Length of fields to compare.  */
   int diff;
 
   if (jf_1 < line1->nfields)
@@ -334,7 +342,7 @@ keycmp (struct line const *line1, struct line const *line2,
     }
   else
     {
-      beg1 = NULL;
+      beg1 = nullptr;
       len1 = 0;
     }
 
@@ -345,7 +353,7 @@ keycmp (struct line const *line1, struct line const *line2,
     }
   else
     {
-      beg2 = NULL;
+      beg2 = nullptr;
       len2 = 0;
     }
 
@@ -369,7 +377,7 @@ keycmp (struct line const *line1, struct line const *line2,
 
   if (diff)
     return diff;
-  return len1 < len2 ? -1 : len1 != len2;
+  return (len1 > len2) - (len1 < len2);
 }
 
 /* Check that successive input lines PREV and CURRENT from input file
@@ -391,13 +399,13 @@ check_order (const struct line *prev,
   if (check_input_order != CHECK_ORDER_DISABLED
       && ((check_input_order == CHECK_ORDER_ENABLED) || seen_unpairable))
     {
-      if (!issued_disorder_warning[whatfile-1])
+      if (!issued_disorder_warning[whatfile - 1])
         {
-          size_t join_field = whatfile == 1 ? join_field_1 : join_field_2;
+          idx_t join_field = whatfile == 1 ? join_field_1 : join_field_2;
           if (keycmp (prev, current, join_field, join_field) > 0)
             {
               /* Exclude any trailing newline. */
-              size_t len = current->buf.length;
+              idx_t len = current->buf.length;
               if (0 < len && current->buf.buffer[len - 1] == '\n')
                 --len;
 
@@ -413,7 +421,7 @@ check_order (const struct line *prev,
 
               /* If we get to here, the message was merely a warning.
                  Arrange to issue it only once per file.  */
-              issued_disorder_warning[whatfile-1] = true;
+              issued_disorder_warning[whatfile - 1] = true;
             }
         }
     }
@@ -428,7 +436,7 @@ reset_line (struct line *line)
 static struct line *
 init_linep (struct line **linep)
 {
-  struct line *line = xcalloc (1, sizeof *line);
+  struct line *line = xzalloc (sizeof *line);
   *linep = line;
   return line;
 }
@@ -473,9 +481,7 @@ get_line (FILE *fp, struct line **linep, int which)
 static void
 free_spareline (void)
 {
-  size_t i;
-
-  for (i = 0; i < ARRAY_CARDINALITY (spareline); i++)
+  for (idx_t i = 0; i < ARRAY_CARDINALITY (spareline); i++)
     {
       if (spareline[i])
         {
@@ -490,7 +496,7 @@ initseq (struct seq *seq)
 {
   seq->count = 0;
   seq->alloc = 0;
-  seq->lines = NULL;
+  seq->lines = nullptr;
 }
 
 /* Read a line from FP and add it to SEQ.  Return true if successful.  */
@@ -500,10 +506,9 @@ getseq (FILE *fp, struct seq *seq, int whichfile)
 {
   if (seq->count == seq->alloc)
     {
-      size_t i;
-      seq->lines = X2NREALLOC (seq->lines, &seq->alloc);
-      for (i = seq->count; i < seq->alloc; i++)
-        seq->lines[i] = NULL;
+      seq->lines = xpalloc (seq->lines, &seq->alloc, 1, -1, sizeof *seq->lines);
+      for (idx_t i = seq->count; i < seq->alloc; i++)
+        seq->lines[i] = nullptr;
     }
 
   if (get_line (fp, &seq->lines[seq->count], whichfile))
@@ -528,8 +533,7 @@ advance_seq (FILE *fp, struct seq *seq, bool first, int whichfile)
 static void
 delseq (struct seq *seq)
 {
-  size_t i;
-  for (i = 0; i < seq->alloc; i++)
+  for (idx_t i = 0; i < seq->alloc; i++)
     {
       freeline (seq->lines[i]);
       free (seq->lines[i]);
@@ -542,13 +546,11 @@ delseq (struct seq *seq)
    'empty_filler' if it is nonempty.  */
 
 static void
-prfield (size_t n, struct line const *line)
+prfield (idx_t n, struct line const *line)
 {
-  size_t len;
-
   if (n < line->nfields)
     {
-      len = line->fields[n].len;
+      idx_t len = line->fields[n].len;
       if (len)
         fwrite (line->fields[n].beg, 1, len, stdout);
       else if (empty_filler)
@@ -561,10 +563,10 @@ prfield (size_t n, struct line const *line)
 /* Output all the fields in line, other than the join field.  */
 
 static void
-prfields (struct line const *line, size_t join_field, size_t autocount)
+prfields (struct line const *line, idx_t join_field, idx_t autocount)
 {
-  size_t i;
-  size_t nfields = autoformat ? autocount : line->nfields;
+  idx_t i;
+  idx_t nfields = autoformat ? autocount : line->nfields;
   char output_separator = tab < 0 ? ' ' : tab;
 
   for (i = 0; i < join_field && i < nfields; ++i)
@@ -586,7 +588,7 @@ prjoin (struct line const *line1, struct line const *line2)
 {
   const struct outlist *outlist;
   char output_separator = tab < 0 ? ' ' : tab;
-  size_t field;
+  idx_t field;
   struct line const *line;
 
   outlist = outlist_head.next;
@@ -595,7 +597,7 @@ prjoin (struct line const *line1, struct line const *line2)
       const struct outlist *o;
 
       o = outlist;
-      while (1)
+      while (true)
         {
           if (o->file == 0)
             {
@@ -617,7 +619,7 @@ prjoin (struct line const *line1, struct line const *line2)
             }
           prfield (field, line);
           o = o->next;
-          if (o == NULL)
+          if (o == nullptr)
             break;
           putchar (output_separator);
         }
@@ -645,6 +647,9 @@ prjoin (struct line const *line1, struct line const *line2)
 
       putchar (eolchar);
     }
+
+  if (ferror (stdout))
+    write_error ();
 }
 
 /* Print the join of the files in FP1 and FP2.  */
@@ -676,8 +681,8 @@ join (FILE *fp1, FILE *fp2)
       struct line const *hline1 = seq1.count ? seq1.lines[0] : &uni_blank;
       struct line const *hline2 = seq2.count ? seq2.lines[0] : &uni_blank;
       prjoin (hline1, hline2);
-      prevline[0] = NULL;
-      prevline[1] = NULL;
+      prevline[0] = nullptr;
+      prevline[1] = nullptr;
       if (seq1.count)
         advance_seq (fp1, &seq1, true, 1);
       if (seq2.count)
@@ -686,7 +691,6 @@ join (FILE *fp1, FILE *fp2)
 
   while (seq1.count && seq2.count)
     {
-      size_t i;
       diff = keycmp (seq1.lines[0], seq2.lines[0],
                      join_field_1, join_field_2);
       if (diff < 0)
@@ -734,9 +738,9 @@ join (FILE *fp1, FILE *fp2)
 
       if (print_pairables)
         {
-          for (i = 0; i < seq1.count - 1; ++i)
+          for (idx_t i = 0; i < seq1.count - 1; ++i)
             {
-              size_t j;
+              idx_t j;
               for (j = 0; j < seq2.count - 1; ++j)
                 prjoin (seq1.lines[i], seq2.lines[j]);
             }
@@ -763,7 +767,7 @@ join (FILE *fp1, FILE *fp2)
      tail ends of both inputs to verify that they are in order.  We
      skip the rest of the tail once we have issued a warning for that
      file, unless we actually need to print the unpairable lines.  */
-  struct line *line = NULL;
+  struct line *line = nullptr;
   bool checktail = false;
 
   if (check_input_order != CHECK_ORDER_DISABLED
@@ -810,17 +814,17 @@ join (FILE *fp1, FILE *fp2)
 /* Add a field spec for field FIELD of file FILE to 'outlist'.  */
 
 static void
-add_field (int file, size_t field)
+add_field (int file, idx_t field)
 {
   struct outlist *o;
 
-  assert (file == 0 || file == 1 || file == 2);
-  assert (file != 0 || field == 0);
+  affirm (file == 0 || file == 1 || file == 2);
+  affirm (file != 0 || field == 0);
 
   o = xmalloc (sizeof *o);
   o->file = file;
   o->field = field;
-  o->next = NULL;
+  o->next = nullptr;
 
   /* Add to the end of the list so the fields are in the right order.  */
   outlist_end->next = o;
@@ -830,25 +834,21 @@ add_field (int file, size_t field)
 /* Convert a string of decimal digits, STR (the 1-based join field number),
    to an integral value.  Upon successful conversion, return one less
    (the zero-based field number).  Silently convert too-large values
-   to SIZE_MAX - 1.  Otherwise, if a value cannot be converted, give a
+   to PTRDIFF_MAX.  Otherwise, if a value cannot be converted, give a
    diagnostic and exit.  */
 
-static size_t
+static idx_t
 string_to_join_field (char const *str)
 {
-  size_t result;
-  unsigned long int val;
-  verify (SIZE_MAX <= ULONG_MAX);
+  intmax_t val;
 
-  strtol_error s_err = xstrtoul (str, NULL, 10, &val, "");
-  if (s_err == LONGINT_OVERFLOW || (s_err == LONGINT_OK && SIZE_MAX < val))
-    val = SIZE_MAX;
-  else if (s_err != LONGINT_OK || val == 0)
+  strtol_error s_err = xstrtoimax (str, nullptr, 10, &val, "");
+  if (s_err == LONGINT_OVERFLOW || (s_err == LONGINT_OK && PTRDIFF_MAX < val))
+    val = PTRDIFF_MAX;
+  else if (s_err != LONGINT_OK || val <= 0)
     error (EXIT_FAILURE, 0, _("invalid field number: %s"), quote (str));
 
-  result = val - 1;
-
-  return result;
+  return val - 1;
 }
 
 /* Convert a single field specifier string, S, to a *FILE_INDEX, *FIELD_INDEX
@@ -856,7 +856,7 @@ string_to_join_field (char const *str)
    If S is valid, return true.  Otherwise, give a diagnostic and exit.  */
 
 static void
-decode_field_spec (const char *s, int *file_index, size_t *field_index)
+decode_field_spec (char const *s, int *file_index, idx_t *field_index)
 {
   /* The first character must be 0, 1, or 2.  */
   switch (s[0])
@@ -882,13 +882,6 @@ decode_field_spec (const char *s, int *file_index, size_t *field_index)
     default:
       error (EXIT_FAILURE, 0,
              _("invalid file number in field spec: %s"), quote (s));
-
-      /* Tell gcc -W -Wall that we can't get beyond this point.
-         This avoids a warning (otherwise legit) that the caller's copies
-         of *file_index and *field_index might be used uninitialized.  */
-      abort ();
-
-      break;
     }
 }
 
@@ -902,7 +895,7 @@ add_field_list (char *str)
   do
     {
       int file_index;
-      size_t field_index;
+      idx_t field_index;
       char const *spec_item = p;
 
       p = strpbrk (p, ", \t");
@@ -918,15 +911,11 @@ add_field_list (char *str)
    more than once to incompatible values.  */
 
 static void
-set_join_field (size_t *var, size_t val)
+set_join_field (ptrdiff_t *var, idx_t val)
 {
-  if (*var != SIZE_MAX && *var != val)
-    {
-      unsigned long int var1 = *var + 1;
-      unsigned long int val1 = val + 1;
-      error (EXIT_FAILURE, 0, _("incompatible join fields %lu, %lu"),
-             var1, val1);
-    }
+  if (0 <= *var && *var != val)
+    error (EXIT_FAILURE, 0,
+           _("incompatible join fields %td, %td"), *var, val);
   *var = val;
 }
 
@@ -965,7 +954,7 @@ add_file_name (char *name, char *names[2],
       switch (operand_status[op0])
         {
         case MUST_BE_OPERAND:
-          error (0, 0, _("extra operand %s"), quote (name));
+          error (0, 0, _("extra operand %s"), quoteaf (name));
           usage (EXIT_FAILURE);
 
         case MIGHT_BE_J1_ARG:
@@ -1025,7 +1014,7 @@ main (int argc, char **argv)
   check_input_order = CHECK_ORDER_DEFAULT;
 
   while ((optc = getopt_long (argc, argv, "-a:e:i1:2:j:o:t:v:z",
-                              longopts, NULL))
+                              longopts, nullptr))
          != -1)
     {
       optc_status = MUST_BE_OPERAND;
@@ -1034,12 +1023,12 @@ main (int argc, char **argv)
         {
         case 'v':
             print_pairables = false;
-            /* Fall through.  */
+            FALLTHROUGH;
 
         case 'a':
           {
-            unsigned long int val;
-            if (xstrtoul (optarg, NULL, 10, &val, "") != LONGINT_OK
+            long int val;
+            if (xstrtol (optarg, nullptr, 10, &val, "") != LONGINT_OK
                 || (val != 1 && val != 2))
               error (EXIT_FAILURE, 0,
                      _("invalid field number: %s"), quote (optarg));
@@ -1170,28 +1159,28 @@ main (int argc, char **argv)
         set_join_field (&join_field_2, i);
       }
 
-  if (join_field_1 == SIZE_MAX)
+  if (join_field_1 < 0)
     join_field_1 = 0;
-  if (join_field_2 == SIZE_MAX)
+  if (join_field_2 < 0)
     join_field_2 = 0;
 
   fp1 = STREQ (g_names[0], "-") ? stdin : fopen (g_names[0], "r");
   if (!fp1)
-    error (EXIT_FAILURE, errno, "%s", g_names[0]);
+    error (EXIT_FAILURE, errno, "%s", quotef (g_names[0]));
   fp2 = STREQ (g_names[1], "-") ? stdin : fopen (g_names[1], "r");
   if (!fp2)
-    error (EXIT_FAILURE, errno, "%s", g_names[1]);
+    error (EXIT_FAILURE, errno, "%s", quotef (g_names[1]));
   if (fp1 == fp2)
     error (EXIT_FAILURE, errno, _("both files cannot be standard input"));
   join (fp1, fp2);
 
   if (fclose (fp1) != 0)
-    error (EXIT_FAILURE, errno, "%s", g_names[0]);
+    error (EXIT_FAILURE, errno, "%s", quotef (g_names[0]));
   if (fclose (fp2) != 0)
-    error (EXIT_FAILURE, errno, "%s", g_names[1]);
+    error (EXIT_FAILURE, errno, "%s", quotef (g_names[1]));
 
   if (issued_disorder_warning[0] || issued_disorder_warning[1])
-    exit (EXIT_FAILURE);
+    error (EXIT_FAILURE, 0, _("input is not in sorted order"));
   else
-    exit (EXIT_SUCCESS);
+    return EXIT_SUCCESS;
 }
