@@ -1,5 +1,5 @@
 /* libstdbuf -- a shared lib to preload to setup stdio buffering for a command
-   Copyright (C) 2009-2013 Free Software Foundation, Inc.
+   Copyright (C) 2009-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,21 +12,27 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Pádraig Brady.  LD_PRELOAD idea from Brian Dessent.  */
 
 #include <config.h>
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdint.h>
 #include "system.h"
-#include "verify.h"
+
+/* Deactivate config.h's "rpl_"-prefixed definitions, since we don't
+   link gnulib here, and the replacements aren't needed.  */
+#undef fprintf
+#undef free
+#undef malloc
+#undef strtoumax
 
 /* Note currently for glibc (2.3.5) the following call does not change
    the buffer size, and more problematically does not give any indication
    that the new size request was ignored:
 
-       setvbuf (stdout, (char*)NULL, _IOFBF, 8192);
+       setvbuf (stdout, nullptr, _IOFBF, 8192);
 
    The ISO C99 standard section 7.19.5.6 on the setvbuf function says:
 
@@ -44,9 +50,9 @@
    buffer allocation as usual.  If it is not zero, then except for
    unbuffered files, the buf argument should point to a buffer at least size
    bytes long; this buffer will be used instead of the current buffer.  (If
-   the size argument is not zero but buf is NULL, a buffer of the given size
+   the size argument is not zero but buf is null, a buffer of the given size
    will be allocated immediately, and released on close.  This is an extension
-   to ANSI C; portable code should use a size of 0 with any NULL buffer.)
+   to ANSI C; portable code should use a size of 0 with any null buffer.)
    --------------------
    Another issue is that on glibc-2.7 the following doesn't buffer
    the first write if it's greater than 1 byte.
@@ -58,10 +64,10 @@
    However I think it's just a buggy implementation due to the various
    inconsistencies with write sizes and subsequent writes.  */
 
-static const char *
+static char const *
 fileno_to_name (const int fd)
 {
-  const char *ret = NULL;
+  char const *ret = nullptr;
 
   switch (fd)
     {
@@ -83,11 +89,11 @@ fileno_to_name (const int fd)
 }
 
 static void
-apply_mode (FILE *stream, const char *mode)
+apply_mode (FILE *stream, char const *mode)
 {
-  char *buf = NULL;
+  char *buf = nullptr;
   int setvbuf_mode;
-  size_t size = 0;
+  uintmax_t size = 0;
 
   if (*mode == '0')
     setvbuf_mode = _IONBF;
@@ -96,27 +102,28 @@ apply_mode (FILE *stream, const char *mode)
   else
     {
       setvbuf_mode = _IOFBF;
-      verify (SIZE_MAX <= ULONG_MAX);
-      size = strtoul (mode, NULL, 10);
-      if (size > 0)
-        {
-          if (!(buf = malloc (size)))   /* will be freed by fclose()  */
-            {
-              /* We could defer the allocation to libc, however since
-                 glibc currently ignores the combination of NULL buffer
-                 with non zero size, we'll fail here.  */
-              fprintf (stderr,
-                       _("failed to allocate a %" PRIuMAX
-                         " byte stdio buffer\n"), (uintmax_t) size);
-              return;
-            }
-        }
-      else
+      char *mode_end;
+      size = strtoumax (mode, &mode_end, 10);
+      if (size == 0 || *mode_end)
         {
           fprintf (stderr, _("invalid buffering mode %s for %s\n"),
                    mode, fileno_to_name (fileno (stream)));
           return;
         }
+
+      buf = size <= SIZE_MAX ? malloc (size) : nullptr;
+      if (!buf)
+        {
+          /* We could defer the allocation to libc, however since
+             glibc currently ignores the combination of null buffer
+             with non zero size, we'll fail here.  */
+          fprintf (stderr,
+                   _("failed to allocate a %" PRIuMAX
+                     " byte stdio buffer\n"),
+                   size);
+          return;
+        }
+      /* buf will be freed by fclose.  */
     }
 
   if (setvbuf (stream, buf, setvbuf_mode, size) != 0)
@@ -127,7 +134,8 @@ apply_mode (FILE *stream, const char *mode)
     }
 }
 
-__attribute__ ((constructor)) static void
+/* Use __attribute to avoid elision of __attribute__ on SUNPRO_C etc.  */
+static void __attribute ((constructor))
 stdbuf (void)
 {
   char *e_mode = getenv ("_STDBUF_E");
