@@ -1,5 +1,5 @@
 /* 'ln' program to create links between files.
-   Copyright (C) 1986-2020 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 
 #include "system.h"
 #include "backupfile.h"
-#include "die.h"
-#include "error.h"
 #include "fcntl-safer.h"
 #include "filenamecat.h"
 #include "file-set.h"
@@ -38,12 +36,6 @@
 #include "yesno.h"
 #include "canonicalize.h"
 
-#ifdef O_PATH
-enum { O_PATHSEARCH = O_PATH };
-#else
-enum { O_PATHSEARCH = O_SEARCH };
-#endif
-
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "ln"
 
@@ -98,22 +90,22 @@ enum { DEST_INFO_INITIAL_CAPACITY = 61 };
 
 static struct option const long_options[] =
 {
-  {"backup", optional_argument, NULL, 'b'},
-  {"directory", no_argument, NULL, 'F'},
-  {"no-dereference", no_argument, NULL, 'n'},
-  {"no-target-directory", no_argument, NULL, 'T'},
-  {"force", no_argument, NULL, 'f'},
-  {"interactive", no_argument, NULL, 'i'},
-  {"suffix", required_argument, NULL, 'S'},
-  {"target-directory", required_argument, NULL, 't'},
-  {"logical", no_argument, NULL, 'L'},
-  {"physical", no_argument, NULL, 'P'},
-  {"relative", no_argument, NULL, 'r'},
-  {"symbolic", no_argument, NULL, 's'},
-  {"verbose", no_argument, NULL, 'v'},
+  {"backup", optional_argument, nullptr, 'b'},
+  {"directory", no_argument, nullptr, 'F'},
+  {"no-dereference", no_argument, nullptr, 'n'},
+  {"no-target-directory", no_argument, nullptr, 'T'},
+  {"force", no_argument, nullptr, 'f'},
+  {"interactive", no_argument, nullptr, 'i'},
+  {"suffix", required_argument, nullptr, 'S'},
+  {"target-directory", required_argument, nullptr, 't'},
+  {"logical", no_argument, nullptr, 'L'},
+  {"physical", no_argument, nullptr, 'P'},
+  {"relative", no_argument, nullptr, 'r'},
+  {"symbolic", no_argument, nullptr, 's'},
+  {"verbose", no_argument, nullptr, 'v'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 /* Return an errno value for a system call that returned STATUS.
@@ -129,7 +121,7 @@ errnoize (int status)
    The result is malloced.  */
 
 static char *
-convert_abs_rel (const char *from, const char *target)
+convert_abs_rel (char const *from, char const *target)
 {
   /* Get dirname to generate paths relative to.  We don't resolve
      the full TARGET as the last component could be an existing symlink.  */
@@ -138,7 +130,7 @@ convert_abs_rel (const char *from, const char *target)
   char *realdest = canonicalize_filename_mode (targetdir, CAN_MISSING);
   char *realfrom = canonicalize_filename_mode (from, CAN_MISSING);
 
-  char *relative_from = NULL;
+  char *relative_from = nullptr;
   if (realdest && realfrom)
     {
       /* Write to a PATH_MAX buffer.  */
@@ -147,7 +139,7 @@ convert_abs_rel (const char *from, const char *target)
       if (!relpath (realfrom, realdest, relative_from, PATH_MAX))
         {
           free (relative_from);
-          relative_from = NULL;
+          relative_from = nullptr;
         }
     }
 
@@ -188,8 +180,8 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
 {
   struct stat source_stats;
   int source_status = 1;
-  char *backup_base = NULL;
-  char *rel_source = NULL;
+  char *backup_base = nullptr;
+  char *rel_source = nullptr;
   int nofollow_flag = logical ? 0 : AT_SYMLINK_NOFOLLOW;
   if (link_errno < 0)
     link_errno = atomic_link (source, destdir_fd, dest_base);
@@ -229,14 +221,14 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
               if (errno != ENOENT)
                 {
                   error (0, errno, _("failed to access %s"), quoteaf (dest));
-                  return false;
+                  goto fail;
                 }
               force = false;
             }
           else if (S_ISDIR (dest_stats.st_mode))
             {
               error (0, 0, _("%s: cannot overwrite directory"), quotef (dest));
-              return false;
+              goto fail;
             }
           else if (seen_file (dest_set, dest, &dest_stats))
             {
@@ -245,7 +237,7 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
               error (0, 0,
                      _("will not overwrite just-created %s with %s"),
                      quoteaf_n (0, dest), quoteaf_n (1, source));
-              return false;
+              goto fail;
             }
           else
             {
@@ -274,7 +266,7 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
                     {
                       error (0, 0, _("%s and %s are the same file"),
                              quoteaf_n (0, source), quoteaf_n (1, dest));
-                      return false;
+                      goto fail;
                     }
                 }
 
@@ -285,7 +277,10 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
                       fprintf (stderr, _("%s: replace %s? "),
                                program_name, quoteaf (dest));
                       if (!yesno ())
-                        return true;
+                        {
+                          free (rel_source);
+                          return false;
+                        }
                     }
 
                   if (backup_type != no_backups)
@@ -299,12 +294,12 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
                         {
                           int rename_errno = errno;
                           free (backup_base);
-                          backup_base = NULL;
+                          backup_base = nullptr;
                           if (rename_errno != ENOENT)
-                           {
+                            {
                               error (0, rename_errno, _("cannot backup %s"),
                                      quoteaf (dest));
-                              return false;
+                              goto fail;
                             }
                           force = false;
                         }
@@ -320,7 +315,7 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
          to link A to B.  But strictly following this has the counterintuitive
          effect of losing the contents of B if A does not exist.  Fortunately,
          POSIX 2008 clarified that an application is free to fail early if it
-         can prove that continuing onwards cannot succeed, so we can try to
+         can prove that continuing onward cannot succeed, so we can try to
          link A to B before blindly unlinking B, thus sometimes attempting to
          link a second time during a successful 'ln -f A B'.
 
@@ -355,7 +350,7 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
           if (backup_base)
             {
               char *backup = backup_base;
-              void *alloc = NULL;
+              void *alloc = nullptr;
               ptrdiff_t destdirlen = dest_base - dest;
               if (0 < destdirlen)
                 {
@@ -397,6 +392,10 @@ do_link (char const *source, int destdir_fd, char const *dest_base,
   free (backup_base);
   free (rel_source);
   return link_errno <= 0;
+
+fail:
+  free (rel_source);
+  return false;
 }
 
 void
@@ -440,7 +439,7 @@ interpreted in relation to its parent directory.\n\
   -n, --no-dereference        treat LINK_NAME as a normal file if\n\
                                 it is a symbolic link to a directory\n\
   -P, --physical              make hard links directly to symbolic links\n\
-  -r, --relative              create symbolic links relative to link location\n\
+  -r, --relative              with -s, create links relative to link location\n\
   -s, --symbolic              make symbolic links instead of hard links\n\
 "), stdout);
       fputs (_("\
@@ -469,9 +468,9 @@ main (int argc, char **argv)
   int c;
   bool ok;
   bool make_backups = false;
-  char const *backup_suffix = NULL;
-  char *version_control_string = NULL;
-  char const *target_directory = NULL;
+  char const *backup_suffix = nullptr;
+  char *version_control_string = nullptr;
+  char const *target_directory = nullptr;
   int destdir_fd;
   bool no_target_directory = false;
   int n_files;
@@ -489,7 +488,8 @@ main (int argc, char **argv)
   symbolic_link = remove_existing_files = interactive = verbose
     = hard_dir_link = false;
 
-  while ((c = getopt_long (argc, argv, "bdfinrst:vFLPS:T", long_options, NULL))
+  while ((c = getopt_long (argc, argv, "bdfinrst:vFLPS:T",
+                           long_options, nullptr))
          != -1)
     {
       switch (c)
@@ -528,16 +528,16 @@ main (int argc, char **argv)
           break;
         case 't':
           if (target_directory)
-            die (EXIT_FAILURE, 0, _("multiple target directories specified"));
+            error (EXIT_FAILURE, 0, _("multiple target directories specified"));
           else
             {
               struct stat st;
               if (stat (optarg, &st) != 0)
-                die (EXIT_FAILURE, errno, _("failed to access %s"),
-                     quoteaf (optarg));
+                error (EXIT_FAILURE, errno, _("failed to access %s"),
+                       quoteaf (optarg));
               if (! S_ISDIR (st.st_mode))
-                die (EXIT_FAILURE, 0, _("target %s is not a directory"),
-                     quoteaf (optarg));
+                error (EXIT_FAILURE, 0, _("target %s is not a directory"),
+                       quoteaf (optarg));
             }
           target_directory = optarg;
           break;
@@ -569,7 +569,7 @@ main (int argc, char **argv)
     }
 
   if (relative && !symbolic_link)
-    die (EXIT_FAILURE, 0, _("cannot do --relative without --symbolic"));
+    error (EXIT_FAILURE, 0, _("cannot do --relative without --symbolic"));
 
   if (!hard_dir_link)
     {
@@ -580,9 +580,9 @@ main (int argc, char **argv)
   if (no_target_directory)
     {
       if (target_directory)
-        die (EXIT_FAILURE, 0,
-             _("cannot combine --target-directory "
-               "and --no-target-directory"));
+        error (EXIT_FAILURE, 0,
+               _("cannot combine --target-directory "
+                 "and --no-target-directory"));
       if (n_files != 2)
         {
           if (n_files < 2)
@@ -629,7 +629,7 @@ main (int argc, char **argv)
               target_directory = d;
             }
           else if (! (n_files == 2 && !target_directory))
-            die (EXIT_FAILURE, err, _("target %s"), quoteaf (d));
+            error (EXIT_FAILURE, err, _("target %s"), quoteaf (d));
         }
     }
 
@@ -654,11 +654,11 @@ main (int argc, char **argv)
           && backup_type != numbered_backups)
         {
           dest_set = hash_initialize (DEST_INFO_INITIAL_CAPACITY,
-                                      NULL,
+                                      nullptr,
                                       triple_hash,
                                       triple_compare,
                                       triple_free);
-          if (dest_set == NULL)
+          if (dest_set == nullptr)
             xalloc_die ();
         }
 
@@ -677,5 +677,5 @@ main (int argc, char **argv)
   else
     ok = do_link (file[0], AT_FDCWD, file[1], file[1], link_errno);
 
-  return ok ? EXIT_SUCCESS : EXIT_FAILURE;
+  main_exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
