@@ -1,5 +1,5 @@
 /* tsort - topological sort.
-   Copyright (C) 1998-2005, 2007-2010 Free Software Foundation, Inc.
+   Copyright (C) 1998-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Mark Kettenis <kettenis@phys.uva.nl>.  */
 
@@ -22,19 +22,17 @@
 
 #include <config.h>
 
-#include <assert.h>
-#include <getopt.h>
 #include <sys/types.h>
 
 #include "system.h"
+#include "assure.h"
 #include "long-options.h"
-#include "error.h"
 #include "fadvise.h"
-#include "quote.h"
 #include "readtokens.h"
 #include "stdio--.h"
+#include "quote.h"
 
-/* The official name of this program (e.g., no `g' prefix).  */
+/* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "tsort"
 
 #define AUTHORS proper_name ("Mark Kettenis")
@@ -49,46 +47,50 @@ struct successor
   struct successor *next;
 };
 
-/* Each string is held in core as the head of a list of successors.  */
+/* Each string is held in memory as the head of a list of successors.  */
 struct item
 {
-  const char *str;
+  char const *str;
   struct item *left, *right;
-  int balance; /* -1, 0, or +1 */
+  signed char balance; /* -1, 0, or +1 */
+  bool printed;
   size_t count;
   struct item *qlink;
   struct successor *top;
 };
 
 /* The head of the sorted list.  */
-static struct item *head = NULL;
+static struct item *head = nullptr;
 
-/* The tail of the list of `zeros', strings that have no predecessors.  */
-static struct item *zeros = NULL;
+/* The tail of the list of 'zeros', strings that have no predecessors.  */
+static struct item *zeros = nullptr;
 
 /* Used for loop detection.  */
-static struct item *loop = NULL;
+static struct item *loop = nullptr;
 
 /* The number of strings to sort.  */
 static size_t n_strings = 0;
-
+
 void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-             program_name);
+    emit_try_help ();
   else
     {
       printf (_("\
 Usage: %s [OPTION] [FILE]\n\
 Write totally ordered list consistent with the partial ordering in FILE.\n\
-With no FILE, or when FILE is -, read standard input.\n\
-\n\
 "), program_name);
+
+      emit_stdin_note ();
+
+      fputs (_("\
+\n\
+"), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
 
   exit (status);
@@ -96,24 +98,17 @@ With no FILE, or when FILE is -, read standard input.\n\
 
 /* Create a new item/node for STR.  */
 static struct item *
-new_item (const char *str)
+new_item (char const *str)
 {
-  struct item *k = xmalloc (sizeof *k);
-
-  k->str = (str ? xstrdup (str): NULL);
-  k->left = k->right = NULL;
-  k->balance = 0;
-
   /* T1. Initialize (COUNT[k] <- 0 and TOP[k] <- ^).  */
-  k->count = 0;
-  k->qlink = NULL;
-  k->top = NULL;
-
+  struct item *k = xzalloc (sizeof *k);
+  if (str)
+    k->str = xstrdup (str);
   return k;
 }
 
 /* Search binary tree rooted at *ROOT for STR.  Allocate a new tree if
-   *ROOT is NULL.  Insert a node/item for STR if not found.  Return
+   *ROOT is null.  Insert a node/item for STR if not found.  Return
    the node/item found/created for STR.
 
    This is done according to Algorithm A (Balanced tree search and
@@ -121,16 +116,14 @@ new_item (const char *str)
    Volume 3/Searching and Sorting, pages 455--457.  */
 
 static struct item *
-search_item (struct item *root, const char *str)
+search_item (struct item *root, char const *str)
 {
   struct item *p, *q, *r, *s, *t;
   int a;
 
-  assert (root);
-
   /* Make sure the tree is not empty, since that is what the algorithm
      below expects.  */
-  if (root->right == NULL)
+  if (root->right == nullptr)
     return (root->right = new_item (str));
 
   /* A1. Initialize.  */
@@ -150,7 +143,7 @@ search_item (struct item *root, const char *str)
       else
         q = p->right;
 
-      if (q == NULL)
+      if (q == nullptr)
         {
           /* A5. Insert.  */
           q = new_item (str);
@@ -162,28 +155,30 @@ search_item (struct item *root, const char *str)
             p->right = q;
 
           /* A6. Adjust balance factors.  */
-          assert (!STREQ (str, s->str));
-          if (strcmp (str, s->str) < 0)
+          a = strcmp (str, s->str);
+          if (a < 0)
             {
               r = p = s->left;
               a = -1;
             }
           else
             {
+              affirm (0 < a);
               r = p = s->right;
               a = 1;
             }
 
           while (p != q)
             {
-              assert (!STREQ (str, p->str));
-              if (strcmp (str, p->str) < 0)
+              int cmp = strcmp (str, p->str);
+              if (cmp < 0)
                 {
                   p->balance = -1;
                   p = p->left;
                 }
               else
                 {
+                  affirm (0 < cmp);
                   p->balance = 1;
                   p = p->right;
                 }
@@ -281,7 +276,7 @@ record_relation (struct item *j, struct item *k)
 }
 
 static bool
-count_items (struct item *unused ATTRIBUTE_UNUSED)
+count_items (MAYBE_UNUSED struct item *unused)
 {
   n_strings++;
   return false;
@@ -291,9 +286,9 @@ static bool
 scan_zeros (struct item *k)
 {
   /* Ignore strings that have already been printed.  */
-  if (k->count == 0 && k->str)
+  if (k->count == 0 && !k->printed)
     {
-      if (head == NULL)
+      if (head == nullptr)
         head = k;
       else
         zeros->qlink = k;
@@ -308,7 +303,7 @@ scan_zeros (struct item *k)
    loop, print the loop on standard error, remove a relation to break
    the loop, and return true.
 
-   The loop detection strategy is as follows: Realise that what we're
+   The loop detection strategy is as follows: Realize that what we're
    dealing with is essentially a directed graph.  If we find an item
    that is part of a graph that contains a cycle we traverse the graph
    in backwards direction.  In general there is no unique way to do
@@ -320,7 +315,7 @@ scan_zeros (struct item *k)
    are stored in the tree is not related to the specified partial
    ordering, we may need to walk the tree several times before the
    loop has completely been constructed.  If the loop was found, the
-   global variable LOOP will be NULL.  */
+   global variable LOOP will be null.  */
 
 static bool
 detect_loop (struct item *k)
@@ -330,7 +325,7 @@ detect_loop (struct item *k)
       /* K does not have to be part of a cycle.  It is however part of
          a graph that contains a cycle.  */
 
-      if (loop == NULL)
+      if (loop == nullptr)
         /* Start traversing the graph at K.  */
         loop = k;
       else
@@ -348,21 +343,22 @@ detect_loop (struct item *k)
                         {
                           struct item *tmp = loop->qlink;
 
-                          fprintf (stderr, "%s: %s\n", program_name,
-                                   loop->str);
+                          error (0, 0, "%s", (loop->str));
 
                           /* Until we encounter K again.  */
                           if (loop == k)
                             {
                               /* Remove relation.  */
-                              (*p)->suc->count--;
-                              *p = (*p)->next;
+                              struct successor *s = *p;
+                              s->suc->count--;
+                              *p = s->next;
+                              IF_LINT (free (s));
                               break;
                             }
 
                           /* Tidy things up since we might have to
                              detect another loop.  */
-                          loop->qlink = NULL;
+                          loop->qlink = nullptr;
                           loop = tmp;
                         }
 
@@ -370,7 +366,7 @@ detect_loop (struct item *k)
                         {
                           struct item *tmp = loop->qlink;
 
-                          loop->qlink = NULL;
+                          loop->qlink = nullptr;
                           loop = tmp;
                         }
 
@@ -400,16 +396,16 @@ detect_loop (struct item *k)
 static bool
 recurse_tree (struct item *root, bool (*action) (struct item *))
 {
-  if (root->left == NULL && root->right == NULL)
+  if (root->left == nullptr && root->right == nullptr)
     return (*action) (root);
   else
     {
-      if (root->left != NULL)
+      if (root->left != nullptr)
         if (recurse_tree (root->left, action))
           return true;
       if ((*action) (root))
         return true;
-      if (root->right != NULL)
+      if (root->right != nullptr)
         if (recurse_tree (root->right, action))
           return true;
     }
@@ -427,51 +423,54 @@ walk_tree (struct item *root, bool (*action) (struct item *))
     recurse_tree (root->right, action);
 }
 
-/* Do a topological sort on FILE.   Return true if successful.  */
+/* Do a topological sort on FILE.  Exit with appropriate exit status.  */
 
-static bool
-tsort (const char *file)
+static _Noreturn void
+tsort (char const *file)
 {
   bool ok = true;
-  struct item *root;
-  struct item *j = NULL;
-  struct item *k = NULL;
+  struct item *j = nullptr;
+  struct item *k = nullptr;
   token_buffer tokenbuffer;
   bool is_stdin = STREQ (file, "-");
 
-  /* Intialize the head of the tree will hold the strings we're sorting.  */
-  root = new_item (NULL);
+  /* Initialize the head of the tree holding the strings we're sorting.  */
+  struct item *root = new_item (nullptr);
 
   if (!is_stdin && ! freopen (file, "r", stdin))
-    error (EXIT_FAILURE, errno, "%s", file);
+    error (EXIT_FAILURE, errno, "%s", quotef (file));
 
   fadvise (stdin, FADVISE_SEQUENTIAL);
 
   init_tokenbuffer (&tokenbuffer);
 
-  while (1)
+  while (true)
     {
       /* T2. Next Relation.  */
       size_t len = readtoken (stdin, DELIM, sizeof (DELIM) - 1, &tokenbuffer);
       if (len == (size_t) -1)
-        break;
+        {
+          if (ferror (stdin))
+            error (EXIT_FAILURE, errno, _("%s: read error"), quotef (file));
+          break;
+        }
 
-      assert (len != 0);
+      affirm (len != 0);
 
       k = search_item (root, tokenbuffer.buffer);
       if (j)
         {
           /* T3. Record the relation.  */
           record_relation (j, k);
-          k = NULL;
+          k = nullptr;
         }
 
       j = k;
     }
 
-  if (k != NULL)
+  if (k != nullptr)
     error (EXIT_FAILURE, 0, _("%s: input contains an odd number of tokens"),
-           file);
+           quotef (file));
 
   /* T1. Initialize (N <- n).  */
   walk_tree (root, count_items);
@@ -487,7 +486,7 @@ tsort (const char *file)
 
           /* T5. Output front of queue.  */
           puts (head->str);
-          head->str = NULL;	/* Avoid printing the same string twice.  */
+          head->printed = true;
           n_strings--;
 
           /* T6. Erase relations.  */
@@ -511,7 +510,7 @@ tsort (const char *file)
       if (n_strings > 0)
         {
           /* The input contains a loop.  */
-          error (0, 0, _("%s: input contains a loop:"), file);
+          error (0, 0, _("%s: input contains a loop:"), quotef (file));
           ok = false;
 
           /* Print the loop and remove a relation to break it.  */
@@ -523,16 +522,14 @@ tsort (const char *file)
 
   if (fclose (stdin) != 0)
     error (EXIT_FAILURE, errno, "%s",
-           is_stdin ? _("standard input") : quote (file));
+           is_stdin ? _("standard input") : quotef (file));
 
-  return ok;
+  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 int
 main (int argc, char **argv)
 {
-  bool ok;
-
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
   setlocale (LC_ALL, "");
@@ -541,10 +538,9 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE, Version,
-                      usage, AUTHORS, (char const *) NULL);
-  if (getopt_long (argc, argv, "", NULL, NULL) != -1)
-    usage (EXIT_FAILURE);
+  parse_gnu_standard_options_only (argc, argv, PROGRAM_NAME, PACKAGE_NAME,
+                                   Version, true, usage, AUTHORS,
+                                   (char const *) nullptr);
 
   if (1 < argc - optind)
     {
@@ -552,7 +548,5 @@ main (int argc, char **argv)
       usage (EXIT_FAILURE);
     }
 
-  ok = tsort (optind == argc ? "-" : argv[optind]);
-
-  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
+  tsort (optind == argc ? "-" : argv[optind]);
 }
