@@ -25,8 +25,6 @@
 #include <sys/types.h>
 
 #include "system.h"
-#include "die.h"
-#include "error.h"
 #include "long-options.h"
 #include "quote.h"
 
@@ -67,7 +65,7 @@ main (int argc, char **argv)
 
   parse_gnu_standard_options_only (argc, argv, PROGRAM_NAME, PACKAGE_NAME,
                                    Version, true, usage, AUTHORS,
-                                   (char const *) NULL);
+                                   (char const *) nullptr);
 
   if (argc < optind + 1)
     {
@@ -82,7 +80,7 @@ main (int argc, char **argv)
     }
 
   if (unlink (argv[optind]) != 0)
-    die (EXIT_FAILURE, errno, _("cannot unlink %s"), quoteaf (argv[optind]));
+    error (EXIT_FAILURE, errno, _("cannot unlink %s"), quoteaf (argv[optind]));
 
   return EXIT_SUCCESS;
 }
