@@ -1,7 +1,7 @@
 #!/bin/sh
 # Test cp backup.
 
-# Copyright (C) 1997-2013 Free Software Foundation, Inc.
+# Copyright (C) 1997-2023 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 . "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
 print_ver_ cp
@@ -23,7 +23,7 @@ suffix=.b
 file=F
 file_backup="$file$suffix"
 
-echo test > $file || fail=1
+echo test > $file || framework_failure_
 
 # Specify both version control and suffix so the environment variables
 # (possibly set by the user running these tests) aren't used.
